% Matlab code example for Algorithm 1 presented in "Sparse Fast DCT for 
% Vectors with Short Support" by Sina Bittens and Gerlind Plonka.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ x, mu, m ] = DCT_shortsupp_bound( dct_x,  epsilon, M )
%DCT_SHORTSUPP_BOUND Reconstructs the vector x with short support from its 
% DCT-II dct_x.

% x needs to be nonnegative, see find(x>epsilon) - should be find(abs(x)>epsilon)

% The length N of dct_x has to be a power of 2. Write it as N=2^J. 
N=length(dct_x);
if abs(log2(N)-round(log2(N)))<eps 
    J=log2(N);
    L=ceil(log2(M))+1;
    % Compute the initial periodized vector x^(L) of length 2^L.
    x=dct(sqrt(2)^(J-L)*dct_x(2^(J-L)*(0:2^L-1)+1),'Type',3);
    % Find the first support index mu and the support length m of x^(L).
    T=find(x>epsilon)-1;
    mu=T(1);
    m=T(end)-T(1)+1;
    % In each step recover x^(j+1) from x^(j) and additional entries of
    % dct_x using the appropriate method for the given support of x^(j).
    for j=L:J-1
        %% Recovery Step for a Periodization without Collision
        if mu<2^j-M
            % If the support of x^(j) is not contained in the last M
            % entries, then there has been no collision and the nonzero
            % entries of x^(j) and x^(j+1) are the same, which means that
            % we only need to find the first support index of x^(j+1).
            
            % Find an oddly indexed nonzero entry of the DCT_II of 
            % (x^(j+1)) by only considering O(m) entries.
            [~,k]=max(abs(dct_x(2^(J-j-1)*(2*(0:m-1)+1)+1)));
            k=k-1;
            dct_x_odd=sqrt(2)^(J-j-1)*dct_x(2^(J-j-1)*(2*k+1)+1);           
            % Compute the 2k+1-st entry of the DCT-II of the vector u that 
            % has x^(j) as its periodization and the same first support 
            % index as x^(j).
            uhat=1/sqrt(2^j)*sum(x((mu:mu+m-1)+1).*(cos((2*k+1)*pi/2^(j+2)*(2*(mu:mu+m-1)+1))).');
            % Initialize the periodization x^(j+1) of double length.
            x_aux=zeros(2^(j+1),1);
            % The support lengths of x^(j) and x^(j+1) are the same, m. If 
            % uhat and dct_x_odd are equal, then the first support indices 
            % of x^(j) and x^(j+1) are the same, otherwise the first 
            % support index mu of x^(j+1) is 2^(j+1)-m-mu.
            if abs(uhat-dct_x_odd)<abs(uhat+dct_x_odd)
                x_aux((mu:mu+m-1)+1)=x((mu:mu+m-1)+1);
            else
                mu=2^(j+1)-m-mu;
                x_aux((mu:mu+m-1)+1)=x(2^(j+1)-1-(mu:mu+m-1)+1);
            end
            x=x_aux;   
            
        %% Recovery Step for a Periodization with Collision
        else 
            % Compute K.
            K=ceil(log2(2^j-mu))+1;
            % Restrict to 2^K equations and variables.
            % Collect the required samples of the DCT-II of x^(j+1).
            b_0=sqrt(2)^(J-j-1)*dct_x(2^(J-K)*(2*(0:2^(K-1)-1)+1)+2^(J-j-1)+1);
            b_1=sqrt(2)^(J-j-1)*dct_x(2^(J-K)*(2*(0:2^(K-1)-1)+1)-2^(J-j-1)+1);
            data=b_0-b_1;
            data=data(end:-1:1);

            z=x(2^j-2^(K-1)+(0:2^(K-1)-1)+1);
            a=sqrt(2^(j-K))*(-1)^(2^(j-K))*(1./cos(pi*(2*(0:2^(K-1)-1)+1)/2^(j+2))).'.*((-1).^(0:2^(K-1)-1)).'.*dct(data,'Type',4);
            % Since x is a nonnegative real vector, we apply a thresholding
            % procedure.
            z_0=0.5*(a(end:-1:1)+z);
            aux=find(z_0<=epsilon);
            z_0(aux)=zeros(size(aux));
            z_1=z-z_0;
            z_1=z_1(end:-1:1);
            % Initialize the periodization x^(j+1) of double length.
            x=zeros(2^(j+1),1);
            x((2^j-2^(K-1):2^j-1)+1)=z_0;
            x((2^j:2^j+2^(K-1)-1)+1)=z_1;
            % Find the first support index mu and the support length m of 
            % x^(j+1).
            T=find(x((2^j-2^(K-1):2^j+2^(K-1)-1)+1)>epsilon)-1+2^j-2^(K-1);
            mu=T(1);
            m=T(end)-T(1)+1;
            x;
        end
    end 
else
    disp('The length of dct_x has to be a power of 2.')
end

end

