% Required for the Matlab code example for Algorithm 1 presented in "Real 
% Sparse Fast DCT for Vectors with Short Support" by Sina Bittens and 
% Gerlind Plonka.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [  ] = TEST_DCT_shortsupp_bound( tests,m,M,N,epsilon, variant )
%TEST_DCT_SHORTSUPP_BOUND Computes the average runtime and reconstruction 
% error of DCT_shortsupp_bound (Algorithm 1 in "Real Sparse Fast DCT for 
% Vectors with Short Support") for exactly known short support and for 
% bounded short support, DCT_shortsupp (Algorithm 2 in "Sparse Fast DCT for 
% Vectors with One-Block Support" by Sina Bittens and Gerlind Plonka) and 
% MATLAB's idct_2 for an input vector of length N with short support.

% If variant==0, the average runtimes of both variants of 
% DCT_shortsupp_bound, DCT_shortsupp and idct are computed for randomly 
% generated noiseless data. If variant==2, the average reconstruction 
% errors for randomly generated noisy input vectors (uniform noise with SNR
% values [0 10 ... 50]) are computed for both variants of 
% DCT_shortsupp_bound, DCT_shortsupp and idct_2.

% epsilon is the threshold for cutting off noise required by 
% DCT_shortsupp_bound and DCT_shortsupp in order to find the support of the
% reconstructed vectors.

% For DCT_shortsupp_bound and for DCT_shortsupp the length of the input 
% vector has to be a power of 2. m and M have to be vectors of the same
% length with m<=M componentwise, where m is the support length of the to
% be created vectors and M the bound on the support length.
if (abs(log2(N)-round(log2(N)))<eps && length(M)==length(m) && sum(m<=M)==length(m) )
    J=log2(N);
    MM=length(M);
    % Initialize the matrix for the average runtimes.
    time=zeros(MM,7);
    time(:,1)=m;
    time(:,2)=M;
    % Define the SNR values for noisy signals.
    SNR=zeros(6,1);
    for l=1:length(SNR)
        SNR(l)=(l-1)*10;
    end
    % Initialize the matrix of reconstruction errors for exact data.
    error_ex=zeros(MM,4);    
    for j=1:MM
        % Initialize the matrix for the average runtimes for exact data.
        time_noise=zeros(length(SNR),5);
        time_noise(:,1)=SNR;
        % Initialize the matrix for the average noisy reconstruction 
        % errors.
        error=zeros(length(SNR),5);
        error(:,1)=SNR;
        % Initialize the matrix for the rate of correct recovery.
        correct_recov=zeros(length(SNR),7);
        correct_recov(:,1)=SNR;
        % Define the cutoff thresholds for noisy data, optimized for
        % N=2^20.
        epsilon_noise(6)=0.05;
        epsilon_noise(5)=0.15;
        epsilon_noise(4)=0.4;
        epsilon_noise(3)=1;
        epsilon_noise(2)=1.8;
        epsilon_noise(1)=2.5;
        for t=1:tests
            if mod(t,10)==1
                t
            end
            % For each test create a random first support index mu for x, 
            % which has to be a vector of length N=2^J with short support.
            mu=randi([0 N-m(j)],1);
            x=zeros(N,1);
            % Create the m(j) support indices.
            supp=mu:mu+m(j)-1; 
            % Generate a random number between 0 and floor((m(j)-2)/2) and
            % create that many zeros inside the support, but not at the 
            % first or last support index.
            suppzeros=randi(floor((m(j)-2)/2)+1)-1; 
            % Select suppzeros elements randomly from 1,...,m-2 and remove
            % the respective entries from supp.
            supp(randperm(m(j)-2,suppzeros)+1)=[];
            % Create the m(j)-suppzeros random non-negative entries of the 
            % support.
            x(supp+1)=rand(m(j)-suppzeros,1)*10;
            % Create the input data.
            dct_x=dct(x,'Type',2);
            
            %% Standard IDCT-II Algorithm
            % Measure the runtime of idct.
            tic;
            S_idct=dct(dct_x,'Type',3);
            time(j,6)=time(j,6)+toc;
            
            %% Numerical Experiments for Exact Input Data
            if variant==0
                %% DCT_shortsupp_bound
                % Measure the runtime of DCT_shortsupp_bound for exactly
                % known support lengths.
                tic;
                [S_bound_ex, ~, ~] =DCT_shortsupp_bound(dct_x,epsilon,m(j));
                time(j,3)=time(j,3)+toc;
                % Measure the runtime of DCT_shortsupp_bound for bounded
                % support lengths.
                tic;
                [S_bound, ~, ~] =DCT_shortsupp_bound(dct_x,epsilon,M(j));
                time(j,4)=time(j,4)+toc;
                
                %% DCT_shortsupp
                % Measure the runtime of DCT_shortsupp.
                tic;
                [S_shortsupp, ~, ~, ~] =DCT_shortsupp(dct_x,epsilon);
                time(j,5)=time(j,5)+toc;
                
                % Compute the reconstruction errors for exact data for the 
                % four methods, given by the norm of the difference 
                % between the original vector x and the returned vectors
                % S_bound_ex, S_bound, S_shortsupp and S_idct.
                error_ex(j,1)=error_ex(j,1)+norm(S_bound_ex-x)/N;
                error_ex(j,2)=error_ex(j,2)+norm(S_bound-x)/N;
                error_ex(j,3)=error_ex(j,3)+norm(S_shortsupp-x)/N;
                error_ex(j,4)=error_ex(j,4)+norm(S_idct-x)/N;
            
            %% Numerical Experiments for Noisy Input Data    
            elseif variant==1
                for n=1:length(SNR)
                    % Create random uniformly distributed noise for the 
                    % input vector dct_x.
                    noise=rand(N,1)-0.5;
                    % Scale the noise according to the current SNR value
                    % SNR(n).
                    noise=norm(dct_x)*10^(-SNR(n)/20)/norm(noise)*noise;
                    % Create noisy input data by addition.
                    dct_x_noise=dct_x+noise;
                    
                    %% Standard IDCT for Noisy Input Data
                    % Measure the runtime of idct for noisy input data.
                    tic;
                    S_idct_noise=dct(dct_x_noise,'Type',3);
                    time_noise(n,5)=time_noise(n,5)+toc;
                    
                    %% DCT_shortsupp_bound for Noisy Input Data
                    % Measure the runtime of DCT_shortsupp_bound for noisy 
                    % input data and exactly known short support.
                    tic;
                    [S_bound_noise_ex, mu_bound_n, m_bound_n]=DCT_shortsupp_bound(dct_x_noise,epsilon_noise(n),m(j));
                    time_noise(n,2)=time_noise(n,2)+toc;
                    % Check whether the support of x was found correctly.
                    % First examine whether the support of x is contained
                    % in the returned support of S_bound_noise_ex, and, if
                    % necessary, increase the counter correct_recov(n,2).
                    if (mu>=mu_bound_n && mu+m(j)-1<=mu_bound_n+m_bound_n-1) 
                        correct_recov(n,2)=correct_recov(n,2)+1;
                        % We also introduce a second counter
                        % correct_recov(n,3) which is only increased if
                        % the returned support length of S_bound_noise_ex 
                        % is at most three times the length of the support 
                        % of x.
                        if (m_bound_n <= 3* m(j) && m_bound_n>=m(j))
                            correct_recov(n,3)=correct_recov(n,3)+1;
                        end
                    end
                    % Measure the runtime of DCT_shortsupp_bound for noisy 
                    % input data and bounded short support.
                    tic;
                    [S_bound_noise, mu_bound_n, m_bound_n]=DCT_shortsupp_bound(dct_x_noise,epsilon_noise(n),M(j));
                    time_noise(n,3)=time_noise(n,3)+toc;
                    % Check whether the support of x was found correctly.
                    % First examine whether the support of x is contained
                    % in the returned support of S_bound_noise, and, if
                    % necessary, increase the counter correct_recov(n,4).
                    if (mu>=mu_bound_n && mu+m(j)-1<=mu_bound_n+m_bound_n-1) 
                        correct_recov(n,4)=correct_recov(n,4)+1;
                        % We also introduce a second counter
                        % correct_recov(n,5) which is only increased if
                        % the returned support length of S_bound_noise is 
                        % at most three times the length of the support 
                        % of x.
                        if (m_bound_n <= 3* m(j) && m_bound_n>=m(j))
                            correct_recov(n,5)=correct_recov(n,5)+1;
                        end
                    end
                    %% DCT_shortsupp for Noisy Input Data
                    % Measure the runtime of DCT_shortsupp for noisy input
                    % data.
                    tic;
                    [S_shortsupp_noise, ~, mu_short_n, m_short_n]=DCT_shortsupp(dct_x_noise,epsilon_noise(n));
                    time_noise(n,4)=time_noise(n,4)+toc;
                    % Check whether the support of x was found correctly.
                    % First examine whether the support of x is contained
                    % in the returned support of S_shortsupp_noise, and, if
                    % necessary, increase the counter correct_recov(n,6).
                    if mu_short_n+m_short_n<=N-1
                        if (mu>=mu_short_n && mu+m(j)-1<=mu_short_n+m_short_n-1) 
                            correct_recov(n,6)=correct_recov(n,6)+1;
                            % We also introduce a second counter
                            % correct_recov(n,7) which is only increased if
                            % the returned support length of
                            % S_shortsupp_noise is at most three times the
                            % length of the support of x.
                            if (m_short_n <= 3* m(j) && m_short_n>=m(j))
                                correct_recov(n,7)=correct_recov(n,7)+1;
                            end
                        end
                    else
                        % If 0, N-1 are contained in the support of
                        % S_shortsupp_noise, the block has to be considered
                        % modulo N.
                        if ((mu>=mu_short_n || mu<=mu_short_n+m_short_n-1-N) && (mu+m(j)-1>=mu_short_n || mu+m(j)-1<=mu_short_n+m_short_n-1-N))
                            correct_recov(n,6)=correct_recov(n,6)+1;
                            if  (m_short_n <= 3* m(j) && m_short_n>=m(j))
                                correct_recov(n,7)=correct_recov(n,7)+1;
                            end
                        end
                    end
                    % Compute the reconstruction errors for noisy data for 
                    % the four methods, given by the norm of the 
                    % difference between the original vector x and the 
                    % returned vectors S_bound_noise_ex, S_bound_noise, 
                    % S_shortsupp_noise and S_idct_noise.
                    error(n,2)=error(n,2)+norm(S_bound_noise_ex-x)/N;
                    error(n,3)=error(n,3)+norm(S_bound_noise-x)/N;
                    error(n,4)=error(n,4)+norm(S_shortsupp_noise-x)/N;
                    error(n,5)=error(n,5)+norm(S_idct_noise-x)/N;
                end
            end
        end
        % Compute the average runtimes and the average reconstruction 
        % errors.
        time(j,7)=N;
        time_noise(:,2:5)=time_noise(:,2:5)/tests
        error(:,2:5)=error(:,2:5)/tests
        correct_recov(:,2:7)=correct_recov(:,2:7)/tests
        % Save the matrix of reconstruction error values in a separate file
        %for each pair of support length m(j) and upper bound M(j).
        if variant ==1
            filename = ['error_DCT_bound_' num2str(m(j)) '_' num2str(M(j)) '.dat'];
            dlmwrite(filename, error(:,:), 'delimiter', '\t');
            % Save the matrix of correct recovery rates in a file.
            filename = ['correctrecov_' num2str(m(j)) '_' num2str(M(j)) '.dat'];
            dlmwrite(filename,correct_recov,'delimiter','\t');
            % Save the matrix of noisy runtime values in a file.
            filename = ['runtimeDCT_bound_noise_' num2str(m(j)) '_' num2str(M(j)) '.dat'];
            dlmwrite(filename,time_noise,'delimiter','\t');
        end
    end
    time(:,3:6)=time(:,3:6)/tests
    error_ex=error_ex/tests
    if variant ==0
        % Save the matrix of runtime values in a file.
        dlmwrite('runtimeDCT_bound.dat',time,'delimiter','\t');
        % Save the matrix of exact errors in a file.
        dlmwrite('error_exact.dat',error_ex,'delimiter','\t');
    end
else
    disp('N has to be a power of 2, m and M have to be vectors of the same length with m<=M componentwise.')
end

end

    


