import numpy as np
import math
from scipy.fftpack import dct


def DCT_real(dct_x, epsilon, M):
    """
    Reconstructs the vector x with short support of length <= M from its DCT-II dct_x using Algorithm 1 in "Real Sparse
    Fast DCT for Vectors with Short Support" by Sina Bittens and Gerlind Plonka.
    :param dct_x: DCT-II of x
    :param epsilon: noise threshold
    :param M: upper bound on the support length m of x
    :return x:
    :return mu: first support index of x
    :return m: support length of x
    """

    dct_x = dct_x.ravel()
    N = dct_x.shape[0]
    # The length N of dct_x has to be a power of 2. Write it as N=2^J.
    if abs(math.log(N, 2) - round(math.log(N, 2))) < np.spacing(1):
        J = round(math.log(N, 2))
        L = round(math.ceil(math.log(M, 2)) + 1)
        # Compute the initial reflected periodization x^(L) of length 2^L.
        x = dct(math.sqrt(2)**(J-L) * dct_x[::2**(J - L)], type=3, norm='ortho')
        # x=dct(sqrt(2)^(J-L)*dct_x(2^(J-L)*(0:2^L-1)+1),'Type',3);
        # Find the first support index mu and the support length m of x^(L).
        aux = np.nonzero(abs(x) > epsilon)
        mu = aux[0][0]
        m = aux[0][-1] - mu + 1
        # In each step recover x^(j+1) from x^(j) and additional entries of dct_x using the appropriate method for the
        # given support of x^(j).
        for j in range(L, J):
            # Recovery step for a reflected periodization without collision
            if mu < 2**j - M:
                # If the support of x^(j) is not contained in the last M entries, then there has been no collision and
                # the nonzero entries of x^(j) and x^(j+1) are the same.

                # Find an oddly indexed nonzero entry of the DCT-II of x^(j+1) by only considering O(m) entries.
                k = np.argmax(abs(dct_x[2**(J - j - 1)::2**(J - j)]))
                dct_x_odd = math.sqrt(2)**(J - j - 1) * dct_x[2**(J - j - 1) * (2*k + 1)]
                # Compute the 2k+1st entry of the DCT-II of the vector u that has x^(j) as its reflected periodization
                # and the same first support index as x^(j).
                uhat = 1/math.sqrt(2**j) * (x[mu:mu + m] * np.cos((2*k+1) * np.pi /
                                                                  2**(j+2) * 2*np.arange(mu, mu + m))).sum()
                # Initialize the reflected periodization x^(j+1) of double length.
                x_aux = np.zeros((2**(j+1)))
                # The support lengths of x^(j) and x^(j+1) are the same, m. If uhat and dct_x_odd are equal, then the
                # first support indices of x^(j) and x^(j+1) are the same, otherwise the first support index mu of
                # x^(j+1) is 2^(j+1)-m-mu.
                if abs(uhat - dct_x_odd) < abs(uhat + dct_x_odd):
                    x_aux[mu:mu + m] = x[mu: mu + m]
                else:
                    mu = 2**(j+1) - m - mu
                    x_aux[mu: mu + m] = np.flip(x[2**(j+1) - mu - m: 2**(j+1) - mu], 0)
                x = x_aux
            # Recovery step for a reflected periodization without collision
            else:
                K = math.ceil(math.log(2**j - mu, 2)) + 1
                # Restrict to 2^L equations and variables.
                # Collect the required samples of the DCT-II of x^(j+1).
                ind_0 = 2**(J - K) * (2*np.arange(2**(K-1)) + 1) + 2**(J - j - 1)
                b_0 = math.sqrt(2)**(J - j - 1) * dct_x[ind_0]
                ind_1 = 2**(J - K) * (2*np.arange(2**(K-1)) + 1) - 2**(J - j - 1)
                b_1 = math.sqrt(2)**(J - j - 1) * dct_x[ind_1]
                data = b_0 - b_1
                data = data[-1::-1]

                z = x[2**j - 2**(K - 1):]
                a = math.sqrt(2**(j - K)) * (-1)**(2**(j - K)) * \
                    (1 / np.cos(np.pi * (2 * np.arange(2**(K-1)) + 1) / 2**(j + 2))) * ((-1)**np.arange(2**(K-1))) * \
                    dct(data, type=4, norm='ortho')
                z_0 = 0.5 * (a[-1::-1] + z)
                aux = np.nonzero(abs(z_0) <= epsilon)
                z_0[aux] = 0
                z_1 = z - z_0
                # Initialize the reflected periodization x^(j+1) of double length.
                x = np.zeros((2**(j+1)))
                x[2**j - 2**(K-1): 2**j] = z_0
                x[2**j: 2**j + 2**(K-1)] = z_1[-1::-1]
                # Find the first support index mu and the support length m of x^(j+1).

                aux = np.nonzero(abs(x[2**j - 2**(K-1): 2**j + 2**(K-1)]) > epsilon)
                mu = aux[0][0] + 2**j - 2**(K-1)
                m = aux[0][-1] + 2**j - 2**(K-1) - mu + 1
        return x, mu, m

    else:
        print('The length of dct_x has to be a power of 2.')
